import type { Either } from '@lokalise/node-core';
import type { ZodSchema } from 'zod';
export type LogFormatter<MessagePayloadSchema> = (message: MessagePayloadSchema) => unknown;
export type BarrierCallbackWithoutMessageType<MessagePayloadSchema extends object> = (message: MessagePayloadSchema) => Promise<boolean>;
export declare const defaultLogFormatter: <MessagePayloadSchema>(message: MessagePayloadSchema) => MessagePayloadSchema;
export type HandlerConfigOptions<MessagePayloadSchema extends object> = {
    messageLogFormatter?: LogFormatter<MessagePayloadSchema>;
    preHandlerBarrier?: BarrierCallbackWithoutMessageType<MessagePayloadSchema>;
};
export declare class MessageHandlerConfig<const MessagePayloadSchema extends object, const ExecutionContext> {
    readonly schema: ZodSchema<MessagePayloadSchema>;
    readonly handler: Handler<MessagePayloadSchema, ExecutionContext>;
    readonly messageLogFormatter: LogFormatter<MessagePayloadSchema>;
    readonly preHandlerBarrier?: BarrierCallbackWithoutMessageType<MessagePayloadSchema>;
    constructor(schema: ZodSchema<MessagePayloadSchema>, handler: Handler<MessagePayloadSchema, ExecutionContext>, options?: HandlerConfigOptions<MessagePayloadSchema>);
}
export declare class MessageHandlerConfigBuilder<MessagePayloadSchemas extends object, ExecutionContext> {
    private readonly configs;
    constructor();
    addConfig<MessagePayloadSchema extends MessagePayloadSchemas>(schema: ZodSchema<MessagePayloadSchema>, handler: Handler<MessagePayloadSchema, ExecutionContext>, options?: HandlerConfigOptions<MessagePayloadSchema>): this;
    build(): MessageHandlerConfig<MessagePayloadSchemas, ExecutionContext>[];
}
export type Handler<MessagePayloadSchemas, ExecutionContext> = (message: MessagePayloadSchemas, context: ExecutionContext) => Promise<Either<'retryLater', 'success'>>;
export type HandlerContainerOptions<MessagePayloadSchemas extends object, ExecutionContext> = {
    messageHandlers: MessageHandlerConfig<MessagePayloadSchemas, ExecutionContext>[];
    messageTypeField: string;
};
export declare class HandlerContainer<MessagePayloadSchemas extends object, ExecutionContext> {
    private readonly messageHandlers;
    private readonly messageTypeField;
    constructor(options: HandlerContainerOptions<MessagePayloadSchemas, ExecutionContext>);
    resolveHandler(messageType: string): MessageHandlerConfig<MessagePayloadSchemas, ExecutionContext>;
    private resolveHandlerMap;
}
